"""testtube settings module."""
import argparse
import os
import sys

# testube settings
SRC_DIR = None
PATTERNS = ()


def get_arguments():
    """Prompts the user for a source directory and an optional settings
    module.

    """
    parser = argparse.ArgumentParser(
        description='Watch a directory and run a custom set of tests whenever'
                    ' a file changes.')
    parser.add_argument(
        '--src_dir', type=str, default=os.getcwd(),
        help='The directory to watch for changes. (Defaults to CWD)')
    parser.add_argument(
        '--settings', type=str, default='tube',
        help='The testtube settings module that defines which tests to run.'
             ' (Defaults to "tube" - your settings module must be importable'
             ' from your current working directory)')
    args = parser.parse_args()

    return args.src_dir, args.settings


def _set_src_dir(src_dir):
    """Generate an absolute path by merging the cwd with the passed src dir"""
    global SRC_DIR

    SRC_DIR = os.path.realpath(os.path.join(os.getcwd(), src_dir))


def _get_test_suite_from_settings(settings_module):
    """Import settings_module and extract the relevant settings."""
    global PATTERNS

    sys.path.append(os.getcwd())
    settings = __import__(settings_module)

    PATTERNS = settings.PATTERNS


def configure(src_dir, settings):
    """Configure the app to use the specified SRC_DIR and extract the
    relevant settings from the specified settings module.

    """
    _set_src_dir(src_dir)
    _get_test_suite_from_settings(settings)



from watchdog.events import FileSystemEventHandler

from testtube.runner import run_tests


class PyChangeHandler(FileSystemEventHandler):
    """Watchdog handler that executes the test runner on file changes."""
    def on_any_event(self, event):
        run_tests(event.src_path)



import re

from testtube import conf


def _inspect_path(path, pattern):
    """Return True if pattern matches path as well as the set of named
    subpattern matches.

    """
    match = re.match(pattern, path)

    if not match:
        return False, {}

    return True, match.groupdict()


def _test_path(path, tests, kwargs):
    """Runs a set of tests against a specified path passing kwargs to each."""
    for test in tests:
        test(path, **kwargs)


def run_tests(path):
    """Checks the given path agains conf.PATTERNS, and if there are any
    matches, it runs the correspond test set against the path.

    """
    for pattern, tests in conf.PATTERNS:
        run_tests, kwargs = _inspect_path(path, pattern)

        if run_tests:
            _test_path(path, tests, kwargs)
            print "=========================================================="



"""Automatically runs a set of tests against a project on file update."""
import time

from watchdog.observers import Observer

from testtube import conf
from testtube.handlers import PyChangeHandler


def main():
    # Configure the app based on passed arguments
    conf.configure(*conf.get_arguments())

    observer = Observer()
    observer.schedule(PyChangeHandler(), conf.SRC_DIR, recursive=True)
    observer.start()

    print "testtube is now watching %s for changes...\n" % conf.SRC_DIR

    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        observer.stop()

    observer.join()



import subprocess

from testtube import conf


def pep8(changed, **kwargs):
    """Runs the pep8 checker against the changed file."""
    _required('pep8')

    print 'Checking PEP 8 compliance of %s...\n' % _short_path(changed)
    subprocess.call(['pep8', changed])
    print '\nDone.\n'


def pep8_all(changed, **kwargs):
    """Runs the pep8 checker against the entire project."""
    _required('pep8')

    print 'Checking PEP 8 compliance of source directory...\n'
    subprocess.call(['pep8', conf.SRC_DIR])
    print '\nDone.\n'


def pyflakes(changed, **kwargs):
    """Runs pyflakes against the changed file"""
    _required('pyflakes')

    print 'Inspecting %s with pyflakes...\n' % _short_path(changed)
    subprocess.call(['pyflakes', changed])
    print '\nDone.\n'


def pyflakes_all(changed, **kwargs):
    """Runs pyflakes against the entire project"""
    _required('pyflakes')

    print 'Inspecting source directory with pyflakes...\n'
    subprocess.call(['pyflakes', conf.SRC_DIR])
    print '\nDone.\n'


def nosetests_all(changed, **kwargs):
    """Run nosetests against the entire project if any file changes."""
    _required('nose')

    print "Running nosetests..."
    subprocess.call(['nosetests'])


def _required(module_name):
    """If the specified module is not available, raise an import error with
    a helpful error.

    """
    try:
        __import__(module_name)
    except:
        raise ImportError(
            '%s must be installed to use this helper.' % module_name)


def _short_path(path):
    """Remove conf.SRC_DIRc from a given path."""
    return path.partition("%s%s" % (conf.SRC_DIR, '/'))[2]



from setuptools import setup, find_packages

from testtube import __version__, __author__


setup(
    name="testtube",
    version=__version__,
    url='https://github.com/thomasw/testtube',
    download_url='https://github.com/thomasw/testtube/downloads',
    author=__author__,
    author_email='thomas.welfley+testtube@gmail.com',
    description='Testtube watches a specified directory for file changes '
                'and runs a set of defined tests against those files whenever'
                ' a change occurs.',
    packages=find_packages(),
    scripts=['testtube/bin/stir'],
    tests_require=["nose==1.2.1", "pinocchio==0.3.1", "unittest2==0.5.1",
                   "mock==1.0.1"],
    install_requires=['watchdog==0.6.0'],
    classifiers=[
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Testing',
    ],
    test_suite='nose.collector',
)
